From: Hui Zhu <hui_zhu@mentor.com>
To: <gdb-patches@sourceware.org>
Subject: [RFC] Autoload-breakpoints new version [3/9] notification async
Date: Tue, 07 Aug 2012 07:10:00 -0000 [thread overview]
Message-ID: <5020BF3A.8030809@mentor.com> (raw)
[-- Attachment #1: Type: text/plain, Size: 1009 bytes --]
This patch is the extend for the notification function.
Current notification function cannot handle the packet when GDB doesn't want to send or receive packet from the remote target.
This patch do the extend make GDB can handle the notification even if it doesn't send or receive packet.
Thanks,
Hui
2012-08-07 Hui Zhu <hui_zhu@mentor.com>
* remote.c (async_client_callback): Move to the top.
(async_client_context): Ditto.
(remote_async_serial_handler): Ditto.
(PACKET_NotificationAsync): New enum.
(remote_pr): Add PACKET_NotificationAsync.
(remote_open_1): Call serial_async if need.
(readchar_buffer_ch): New variable.
(readchar): If need, return the value of readchar_buffer_ch.
(readchar_buffer_put): New function.
(remote_is_async_p): Add check for PACKET_ReportAsync.
(remote_notificationasync_check): New function.
(remote_async_serial_handler): Add code to handle notification.
(remote_async): Add check for PACKET_ReportAsync.
(_initialize_remote): Add PACKET_NotificationAsync.
[-- Warning: decoded text below may be mangled, UTF-8 assumed --]
[-- Attachment #2: notification_async.txt --]
[-- Type: text/plain; charset="us-ascii"; name="notification_async.txt", Size: 5920 bytes --]
---
remote.c | 108 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++------
1 file changed, 98 insertions(+), 10 deletions(-)
--- a/remote.c
+++ b/remote.c
@@ -224,6 +224,11 @@ static void remote_query_supported (void
static void remote_check_symbols (struct objfile *objfile);
+static void (*async_client_callback) (enum inferior_event_type event_type,
+ void *context);
+static void *async_client_context;
+static serial_event_ftype remote_async_serial_handler;
+
void _initialize_remote (void);
struct stop_reply;
@@ -1297,6 +1302,7 @@ enum {
PACKET_QDisableRandomization,
PACKET_QAgent,
PACKET_AutoloadBreakpoints,
+ PACKET_NotificationAsync,
PACKET_MAX
};
@@ -3998,6 +4004,8 @@ static struct protocol_feature remote_pr
remote_string_tracing_feature, -1 },
{ "AutoloadBreakpoints", PACKET_DISABLE, remote_supported_packet,
PACKET_AutoloadBreakpoints },
+ { "NotificationAsync", PACKET_DISABLE, remote_supported_packet,
+ PACKET_NotificationAsync },
};
static char *remote_support_xml;
@@ -4355,6 +4363,10 @@ remote_open_1 (char *name, int from_tty,
if (target_async_permitted)
wait_forever_enabled_p = 1;
+
+ if (remote_protocol_packets[PACKET_NotificationAsync].support
+ != PACKET_DISABLE)
+ serial_async (remote_desc, remote_async_serial_handler, NULL);
}
/* This takes a program previously attached to and detaches it. After
@@ -6980,11 +6992,20 @@ remote_files_info (struct target_ops *ig
/* Read a single character from the remote end. */
+static int readchar_buffer_ch = -1;
+
static int
readchar (int timeout)
{
int ch;
+ if (readchar_buffer_ch != -1)
+ {
+ ch = readchar_buffer_ch;
+ readchar_buffer_ch = -1;
+ return ch;
+ }
+
ch = serial_readchar (remote_desc, timeout);
if (ch >= 0)
@@ -7007,6 +7028,17 @@ readchar (int timeout)
return ch;
}
+/* When the function that call the readchar got a char is not for it
+ and other function need this charï¼call this function to put
+ this char back. Then when other function call readchar, it will
+ got this char first. */
+
+static void
+readchar_buffer_put (int ch)
+{
+ readchar_buffer_ch = ch;
+}
+
/* Send the command in *BUF to the remote machine, and read the reply
into *BUF. Report an error if we get an error reply. Resize
*BUF using xrealloc if necessary to hold the result, and update
@@ -11207,25 +11239,71 @@ remote_is_async_p (void)
/* We only enable async when the user specifically asks for it. */
return 0;
+ if (remote_protocol_packets[PACKET_NotificationAsync].support
+ != PACKET_DISABLE)
+ return async_client_callback != NULL;
+
/* We're async whenever the serial device is. */
return serial_is_async_p (remote_desc);
}
+static int
+remote_notificationasync_check (void)
+{
+ int c = readchar (-1);
+
+ if (c == '%')
+ return 1;
+
+ readchar_buffer_put (c);
+ return 0;
+}
+
/* Pass the SERIAL event on and up to the client. One day this code
will be able to delay notifying the client of an event until the
point where an entire packet has been received. */
-static void (*async_client_callback) (enum inferior_event_type event_type,
- void *context);
-static void *async_client_context;
-static serial_event_ftype remote_async_serial_handler;
-
static void
remote_async_serial_handler (struct serial *scb, void *context)
{
- /* Don't propogate error information up to the client. Instead let
- the client find out about the error by querying the target. */
- async_client_callback (INF_REG_EVENT, async_client_context);
+ /* Check if this is ReportAsync. */
+ if (remote_notificationasync_check ())
+ {
+ int val;
+ struct remote_state *rs = get_remote_state ();
+
+ val = read_frame (&rs->buf, &rs->buf_size);
+ if (val >= 0)
+ {
+ if (remote_debug)
+ {
+ struct cleanup *old_chain;
+ char *str;
+
+ str = escape_buffer (rs->buf, val);
+ old_chain = make_cleanup (xfree, str);
+ fprintf_unfiltered (gdb_stdlog, " Notification received: %s\n",
+ str);
+ do_cleanups (old_chain);
+ }
+ handle_notification (rs->buf, val);
+ }
+ else
+ {
+ if (remote_debug)
+ {
+ fprintf_unfiltered (gdb_stdlog, "putpkt: Junk: ");
+ fprintf_unfiltered (gdb_stdlog, "%%%s", rs->buf);
+ }
+ }
+ }
+ else
+ {
+ /* Don't propogate error information up to the client. Instead let
+ the client find out about the error by querying the target. */
+ if (async_client_callback)
+ async_client_callback (INF_REG_EVENT, async_client_context);
+ }
}
static void
@@ -11246,12 +11324,19 @@ remote_async (void (*callback) (enum inf
{
if (callback != NULL)
{
- serial_async (remote_desc, remote_async_serial_handler, NULL);
+ if (remote_protocol_packets[PACKET_NotificationAsync].support
+ == PACKET_DISABLE)
+ serial_async (remote_desc, remote_async_serial_handler, NULL);
async_client_callback = callback;
async_client_context = context;
}
else
- serial_async (remote_desc, NULL, NULL);
+ {
+ if (remote_protocol_packets[PACKET_NotificationAsync].support
+ == PACKET_DISABLE)
+ serial_async (remote_desc, NULL, NULL);
+ async_client_callback = NULL;
+ }
}
static void
@@ -11769,6 +11854,9 @@ Show the maximum size of the address (in
add_packet_config_cmd (&remote_protocol_packets[PACKET_AutoloadBreakpoints],
"AutoloadBreakpoints", "autoload-breakpoints", 0);
+ add_packet_config_cmd (&remote_protocol_packets[PACKET_NotificationAsync],
+ "NotificationAsync", "notification-async", 0);
+
/* Keep the old ``set remote Z-packet ...'' working. Each individual
Z sub-packet has its own set and show commands, but users may
have sets to this variable in their .gdbinit files (or in their
next reply other threads:[~2012-08-07 7:10 UTC|newest]
Thread overview: 10+ messages / expand[flat|nested] mbox.gz Atom feed top
2012-08-07 7:10 Hui Zhu [this message]
2012-08-08 13:26 ` Yao Qi
2012-08-11 15:20 ` Hui Zhu
2012-08-13 2:25 ` Yao Qi
2012-08-14 12:01 ` Hui Zhu
2012-08-15 14:23 ` Hui Zhu
2012-08-23 10:44 ` Hui Zhu
2012-08-23 11:20 ` Pedro Alves
2012-08-23 12:46 ` Yao Qi
2012-08-23 14:30 ` Hui Zhu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=5020BF3A.8030809@mentor.com \
--to=hui_zhu@mentor.com \
--cc=gdb-patches@sourceware.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox